function loadCss() {
  const styles = `
    @charset "UTF-8";
    .am-notif-container {
      position: fixed;
      margin: 10px;
      width: 250px;
      display: flex;
      flex-direction: column;
      gap: 0.5rem;
    }
    .am-notif-container .am-notif-content {
      box-sizing: border-box;
      padding: 0.9rem;
      background-color: var(--color);
      border-radius: 0.5rem;
      position: relative;
      cursor: pointer;
      transition: transform 300ms ease-in-out;
      box-shadow: 0 10px 15px -3px rgba(0, 0, 0, 0.1), 0 4px 6px -2px rgba(0, 0, 0, 0.05);
    }
    .am-notif-container .am-notif-content.show-progress .am-notif-head {
      background: conic-gradient(var(--color) var(--progress), #fff 0%);
      scroll-behavior: smooth;
    }
    .am-notif-container .am-notif-content .am-notif-head {
      width: 30px;
      height: 30px;
      background: #fff;
      position: absolute;
      top: -15px;
      left: -15px;
      border-radius: 100%;
      box-shadow: 0 10px 15px -3px rgba(0, 0, 0, 0.1), 0 4px 6px -2px rgba(0, 0, 0, 0.05);
    }
    .am-notif-container .am-notif-content .am-notif-head .am-notif-head-content {
      position: relative;
      width: 100%;
      height: 100%;
      display: grid;
      place-items: center;
    }
    .am-notif-container .am-notif-content .am-notif-head .am-notif-head-content .time {
      background-color: #fff;
      height: calc(100% - 5px);
      width: calc(100% - 5px);
      border-radius: 50%;
      display: grid;
      place-items: center;
    }
    .am-notif-container .am-notif-content .am-notif-head .am-notif-head-content .time span {
      color: var(--color);
    }
    .am-notif-container .am-notif-content .am-notif-head .am-notif-head-content .time span::after {
      content: var(--symbol);
      font-size: 22px;
      position: relative;
      top: -4px;
    }
    @keyframes decreaseTime {
      to {
        transform: rotate(1turn);
      }
    }
    .am-notif-container .am-notif-content .am-notif-body {
      overflow: hidden;
      width: 100%;
      height: 100%;
    }
    .am-notif-container .am-notif-content .am-notif-body .alert-text {
      color: #fff;
      display: flex;
      align-items: start;
      justify-content: space-between;
      gap: 0.5rem;
      font-size: 0.875rem;
      font-weight: 500;
    }
    .am-notif-container .am-notif-content .am-notif-body .progress {
      position: absolute;
      left: 0;
      bottom: 0;
      width: calc(100% - 30px);
      height: 20px;
    }
    .am-notif-container .am-notif-content .am-notif-body .progress::before {
      content: "";
      position: absolute;
      height: 2px;
      width: calc(100% * var(--progress));
      background-color: #fff;
      bottom: 0;
      left: 0;
      right: 0;
    }
    .am-notif-container .am-notif-content.show {
      transform: translate(0, 0);
    }
    .am-notif-container .am-notif-content.can-close::after {
      content: "×";
      position: absolute;
      top: -2px;
      right: 5px;
      font-size: 1.1rem;
      color: #fff;
    }
    .am-notif-container .am-notif-content.progress::before {
      content: "";
      position: absolute;
      height: 2px;
      width: calc(100% * var(--progress));
      background-color: blue;
      bottom: 0;
      left: 0;
      right: 0;
    }
    
    .am-notif-container[data-position$=-right] .am-notif-content {
      transform: translateX(110%);
    }
    
    .am-notif-container[data-position$=-left] .am-notif-content {
      transform: translateX(-110%);
    }
    
    .am-notif-container[data-position=top-center] .am-notif-content {
      transform: translateY(-100vh);
    }
    
    .am-notif-container[data-position=bottom-center] .am-notif-content {
      transform: translateY(100vh);
    }
    
    .am-notif-container .am-notif-content.show {
      transform: translate(0, 0);
    }
    
    .am-notif-content.progress::before {
      content: "";
      position: absolute;
      height: 2px;
      width: calc(100% * var(--progress));
      background-color: blue;
      bottom: 0;
      left: 0;
      right: 0;
    }
    
    .am-notif-container[data-position^=top-] {
      top: 0;
    }
    
    .am-notif-container[data-position^=bottom-] {
      bottom: 0;
    }
    
    .am-notif-container[data-position$=-right] {
      right: 0;
    }
    
    .am-notif-container[data-position$=-left] {
      left: 0;
    }
    
    .am-notif-container[data-position$=-center] {
      left: 50%;
      transform: translateX(-50%);
    }
  `;

  const styleElem = document.createElement('style');
  styleElem.textContent = styles;
  document.head.appendChild(styleElem);
}

const DEFAULT_OPTIONS = {
  autoClose: 5000,
  position: "top-right",
  onClose: () => {},
  canClose: true,
  showProgress: true,
};

export default class AmNotif {
  #amNotifElem;
  #autoCloseInterval;
  #progressInterval;
  #removeBinded;
  #timeVisible = 0;
  #autoClose;
  #isPaused = false;
  #unpause;
  #pause;
  #visibilityChange;
  #shouldUnPause;

  constructor(options) {
    this.#amNotifElem = document.createElement("div");
    this.#amNotifElem.classList.add("am-notif-content");
    let amNotifContent = this.#amNotifElem;
    var amNotifHead = document.createElement('div');
    var amNotifHeadContent = document.createElement('div');
    var amNotifHeadTime = document.createElement('div');
    var amNotifHeadIcon = document.createElement('span');
    let amNotifBody = document.createElement("div");
    amNotifHead.classList.add("am-notif-head");
    amNotifHeadContent.classList.add("am-notif-head-content");
    amNotifHeadTime.classList.add("time");
    amNotifBody.classList.add("am-notif-body");
    amNotifHead.appendChild(amNotifHeadContent);
    amNotifHeadContent.appendChild(amNotifHeadTime);
    amNotifHeadTime.appendChild(amNotifHeadIcon);
    amNotifContent.appendChild(amNotifHead);
    amNotifContent.appendChild(amNotifBody);
    loadCss();
    
    requestAnimationFrame(() => {
      this.#amNotifElem.classList.add("show");
    });
    this.#removeBinded = this.remove.bind(this);
    this.#unpause = () => (this.#isPaused = false);
    this.#pause = () => (this.#isPaused = true);
    this.#visibilityChange = () => {
      this.#shouldUnPause = document.visibilityState === "visible";
    };
    this.update({ ...DEFAULT_OPTIONS, ...options });
  }

  set autoClose(value) {
    this.#autoClose = value;
    this.#timeVisible = 0;
    if (value === false) return;

    let lastTime;
    const func = (time) => {
      if (this.#shouldUnPause) {
        lastTime = null;
        this.#shouldUnPause = false;
      }
      if (lastTime == null) {
        lastTime = time;
        this.#autoCloseInterval = requestAnimationFrame(func);
        return;
      }
      if (!this.#isPaused) {
        this.#timeVisible += time - lastTime;
        if (this.#timeVisible >= this.#autoClose) {
          this.remove();
          return;
        }
      }

      lastTime = time;
      this.#autoCloseInterval = requestAnimationFrame(func);
    };

    this.#autoCloseInterval = requestAnimationFrame(func);
  }

  set position(value) {
    const currentContainer = this.#amNotifElem.parentElement;
    const selector = `.am-notif-container[data-position="${value}"]`;
    const container = document.querySelector(selector) || createContainer(value);
    container.append(this.#amNotifElem);
    if (currentContainer == null || currentContainer.hasChildNodes()) return;
    currentContainer.remove();
  }

  set type(value) {
    let color, symbol;

    if (value == 'success') {
      color = '#32C682';
      symbol = '✓';
    } else if (value == 'danger') {
      color = '#FF5549';
      symbol = '×';
    } else if (value == 'warning') {
      color = '#EEBF31';
      symbol = '!';
    } else {
      color = '#00d4ff';
      symbol = '?';
    }
    
    this.#amNotifElem.style.setProperty('--color', color);
    this.#amNotifElem.style.setProperty('--symbol', `'${symbol}'`);
  }

  set text(value) {
    let amNotifBody = this.#amNotifElem.querySelector('.am-notif-body');
    let alertText = document.createElement('span');
    amNotifBody.appendChild(alertText);

    alertText.className = 'alert-text';
    alertText.textContent = value;
  }

  set canClose(value) {
    this.#amNotifElem.classList.toggle("can-close", value);
    if (value) {
      this.#amNotifElem.addEventListener("click", this.#removeBinded);
    } else {
      this.#amNotifElem.removeEventListener("click", this.#removeBinded);
    }
  }

  set showProgress(value) {
    if (value) {
      const func = () => {
        if (!this.#isPaused) {
          this.#amNotifElem.style.setProperty('--progress', `${100-(Math.round((1 - this.#timeVisible / this.#autoClose)*100))}%`);
          this.#amNotifElem.classList.add('show-progress')
        }
        this.#progressInterval = requestAnimationFrame(func);
      };

      this.#progressInterval = requestAnimationFrame(func);
    }
  }

  set pauseOnHover(value) {
    if (value) {
      this.#amNotifElem.addEventListener("mouseover", this.#pause);
      this.#amNotifElem.addEventListener("mouseleave", this.#unpause);
    } else {
      this.#amNotifElem.removeEventListener("mouseover", this.#pause);
      this.#amNotifElem.removeEventListener("mouseleave", this.#unpause);
    }
  }

  set pauseOnFocusLoss(value) {
    if (value) {
      document.addEventListener("visibilitychange", this.#visibilityChange);
    } else {
      document.removeEventListener("visibilitychange", this.#visibilityChange);
    }
  }

  update(options) {
    Object.entries(options).forEach(([key, value]) => {
      this[key] = value;
    });
  }

  remove() {
    cancelAnimationFrame(this.#autoCloseInterval);
    cancelAnimationFrame(this.#progressInterval);
    const container = this.#amNotifElem.parentElement;
    this.#amNotifElem.classList.remove("show");
    this.#amNotifElem.addEventListener("transitionend", () => {
      this.#amNotifElem.remove();
      if (container.hasChildNodes()) return;
      container.remove();
    });
    this.onClose();
  }
}

function createContainer(position) {
  const container = document.createElement("div");
  container.classList.add("am-notif-container");
  container.dataset.position = position;
  document.body.append(container);
  return container;
}
